@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
 #
 # Copyright 2007-2010 Zuza Software Foundation
 #
@@ -64,16 +62,16 @@ Header
 Escaping
     Wordfast TM implements a form of escaping that covers two aspects:
 
-    1. Placeable: bold, formating, etc.  These are left as is and ignored.
-    It is up to the editor and future placeable implementation to manage these.
+    1. Placeable: bold, formating, etc.  These are left as is and ignored.  It
+       is up to the editor and future placeable implementation to manage these.
 
-    2. Escapes: items that may confuse Excel or translators are
-    escaped as &'XX;. These are fully implemented and are converted to and from
-    Unicode.  By observing behaviour and reading documentation we where able
-    to observe all possible escapes. Unfortunately the escaping differs slightly
-    between Windows and Mac version.  This might cause errors in future.
-    Functions allow for ``<_wf_to_char>`` and back to Wordfast escape
-    (``<_char_to_wf>``).
+    2. Escapes: items that may confuse Excel or translators are escaped as
+       ``&'XX;``. These are fully implemented and are converted to and from
+       Unicode.  By observing behaviour and reading documentation we where able
+       to observe all possible escapes. Unfortunately the escaping differs
+       slightly between Windows and Mac version.  This might cause errors in
+       future.  Functions allow for ``<_wf_to_char>`` and back to Wordfast
+       escape (``<_char_to_wf>``).
 
 Extended Attributes
     The last 4 columns allow users to define and manage extended attributes.
@@ -81,35 +79,60 @@ Extended Attributes
 """
 
 import csv
-import sys
 import time
 
 from translate.storage import base
 
+
 WF_TIMEFORMAT = "%Y%m%d~%H%M%S"
 """Time format used by Wordfast"""
 
-WF_FIELDNAMES_HEADER = ["date", "userlist", "tucount", "src-lang", "version",
-                        "target-lang", "license", "attr1list", "attr2list",
-                        "attr3list", "attr4list", "attr5list"]
+WF_FIELDNAMES_HEADER = [
+    "date",
+    "userlist",
+    "tucount",
+    "src-lang",
+    "version",
+    "target-lang",
+    "license",
+    "attr1list",
+    "attr2list",
+    "attr3list",
+    "attr4list",
+    "attr5list",
+]
 """Field names for the Wordfast header"""
 
-WF_FIELDNAMES = ["date", "user", "reuse", "src-lang", "source", "target-lang",
-                 "target", "attr1", "attr2", "attr3", "attr4"]
+WF_FIELDNAMES = [
+    "date",
+    "user",
+    "reuse",
+    "src-lang",
+    "source",
+    "target-lang",
+    "target",
+    "attr1",
+    "attr2",
+    "attr3",
+    "attr4",
+    "attr5",
+]
 """Field names for a Wordfast TU"""
 
 WF_FIELDNAMES_HEADER_DEFAULTS = {
-"date": "%19000101~121212",
-"userlist": "%User ID,TT,TT Translate-Toolkit",
-"tucount": "%TU=00000001",
-"src-lang": "%EN-US",
-"version": "%Wordfast TM v.5.51w9/00",
-"target-lang": "",
-"license": "%---00000001",
-"attr1list": "",
-"attr2list": "",
-"attr3list": "",
-"attr4list": ""}
+    "date": "%19000101~121212",
+    "userlist": "%User ID,TT,TT Translate-Toolkit",
+    "tucount": "%TU=00000001",
+    "src-lang": "%EN-US",
+    "version": "%Wordfast TM v.5.51w9/00",
+    "target-lang": "",
+    "license": "%---00000001",
+    "attr1list": "",
+    "attr2list": "",
+    "attr3list": "",
+    "attr4list": "",
+    "attr5list": "",
+}
 """Default or minimum header entries for a Wordfast file"""
 
 # TODO Needs validation.  The following need to be checked against a WF TM file
@@ -117,60 +140,61 @@ WF_FIELDNAMES_HEADER_DEFAULTS = {
 # For now these look correct and have been taken from Windows CP1252 and
 # Macintosh code points found for the respective character sets on Linux.
 WF_ESCAPE_MAP = (
-              ("&'26;", u"\u0026"),  # & - Ampersand (must be first to prevent
-                                     #     escaping of escapes)
-              ("&'82;", u"\u201A"),  # ‚ - Single low-9 quotation mark
-              ("&'85;", u"\u2026"),  # … - Elippsis
-              ("&'91;", u"\u2018"),  # ‘ - left single quotation mark
-              ("&'92;", u"\u2019"),  # ’ - right single quotation mark
-              ("&'93;", u"\u201C"),  # “ - left double quotation mark
-              ("&'94;", u"\u201D"),  # ” - right double quotation mark
-              ("&'96;", u"\u2013"),  # – - en dash (validate)
-              ("&'97;", u"\u2014"),  # — - em dash (validate)
-              ("&'99;", u"\u2122"),  # ™ - Trade mark
-              # Windows only
-              ("&'A0;", u"\u00A0"),  #   - Non breaking space
-              ("&'A9;", u"\u00A9"),  # © - Copyright
-              ("&'AE;", u"\u00AE"),  # ® - Registered
-              ("&'BC;", u"\u00BC"),  # ¼
-              ("&'BD;", u"\u00BD"),  # ½
-              ("&'BE;", u"\u00BE"),  # ¾
-              # Mac only
-              ("&'A8;", u"\u00AE"),  # ® - Registered
-              ("&'AA;", u"\u2122"),  # ™ - Trade mark
-              ("&'C7;", u"\u00AB"),  # « - Left-pointing double angle quotation mark
-              ("&'C8;", u"\u00BB"),  # » - Right-pointing double angle quotation mark
-              ("&'C9;", u"\u2026"),  # … - Horizontal Elippsis
-              ("&'CA;", u"\u00A0"),  #   - Non breaking space
-              ("&'D0;", u"\u2013"),  # – - en dash (validate)
-              ("&'D1;", u"\u2014"),  # — - em dash (validate)
-              ("&'D2;", u"\u201C"),  # “ - left double quotation mark
-              ("&'D3;", u"\u201D"),  # ” - right double quotation mark
-              ("&'D4;", u"\u2018"),  # ‘ - left single quotation mark
-              ("&'D5;", u"\u2019"),  # ’ - right single quotation mark
-              ("&'E2;", u"\u201A"),  # ‚ - Single low-9 quotation mark
-              ("&'E3;", u"\u201E"),  # „ - Double low-9 quotation mark
-              # Other markers
-              #("&'B;", u"\n"), # Soft-break - XXX creates a problem with
-                                # roundtripping could also be represented
-                                # by \u2028
-             )
+    ("&'26;", "\u0026"),  # & - Ampersand (must be first to prevent
+    #     escaping of escapes)
+    ("&'82;", "\u201A"),  # ‚ - Single low-9 quotation mark
+    ("&'85;", "\u2026"),  # … - Elippsis
+    ("&'91;", "\u2018"),  # ‘ - left single quotation mark
+    ("&'92;", "\u2019"),  # ’ - right single quotation mark
+    ("&'93;", "\u201C"),  # “ - left double quotation mark
+    ("&'94;", "\u201D"),  # ” - right double quotation mark
+    ("&'96;", "\u2013"),  # – - en dash (validate)
+    ("&'97;", "\u2014"),  # — - em dash (validate)
+    ("&'99;", "\u2122"),  # ™ - Trade mark
+    # Windows only
+    ("&'A0;", "\u00A0"),  #   - Non breaking space
+    ("&'A9;", "\u00A9"),  # © - Copyright
+    ("&'AE;", "\u00AE"),  # ® - Registered
+    ("&'BC;", "\u00BC"),  # ¼
+    ("&'BD;", "\u00BD"),  # ½
+    ("&'BE;", "\u00BE"),  # ¾
+    # Mac only
+    ("&'A8;", "\u00AE"),  # ® - Registered
+    ("&'AA;", "\u2122"),  # ™ - Trade mark
+    ("&'C7;", "\u00AB"),  # « - Left-pointing double angle quotation mark
+    ("&'C8;", "\u00BB"),  # » - Right-pointing double angle quotation mark
+    ("&'C9;", "\u2026"),  # … - Horizontal Elippsis
+    ("&'CA;", "\u00A0"),  #   - Non breaking space
+    ("&'D0;", "\u2013"),  # – - en dash (validate)
+    ("&'D1;", "\u2014"),  # — - em dash (validate)
+    ("&'D2;", "\u201C"),  # “ - left double quotation mark
+    ("&'D3;", "\u201D"),  # ” - right double quotation mark
+    ("&'D4;", "\u2018"),  # ‘ - left single quotation mark
+    ("&'D5;", "\u2019"),  # ’ - right single quotation mark
+    ("&'E2;", "\u201A"),  # ‚ - Single low-9 quotation mark
+    ("&'E3;", "\u201E"),  # „ - Double low-9 quotation mark
+    # Other markers
+    # Soft-break - XXX creates a problem with roundtripping could
+    # also be represented by \u2028
+    # ("&'B;", "\n"),
+)
 """Mapping of Wordfast &'XX; escapes to correct Unicode characters"""
 
-TAB_UTF16 = "\x00\x09"
+TAB_UTF16 = b"\x00\x09"
 """The tab \\t character as it would appear in UTF-16 encoding"""
 
 
 def _char_to_wf(string):
     """Char -> Wordfast &'XX; escapes
 
-       Full roundtripping is not possible because of the escaping of
-       NEWLINE \\n and TAB \\t"""
+    Full roundtripping is not possible because of the escaping of
+    NEWLINE \\n and TAB \\t
+    """
     # FIXME there is no platform check to ensure that we use Mac encodings
     # when running on a Mac
     if string:
         for code, char in WF_ESCAPE_MAP:
-            string = string.replace(char.encode('utf-8'), code)
+            string = string.replace(char, code)
         string = string.replace("\n", "\\n").replace("\t", "\\t")
     return string
 
@@ -179,27 +203,30 @@ def _wf_to_char(string):
     """Wordfast &'XX; escapes -> Char"""
     if string:
         for code, char in WF_ESCAPE_MAP:
-            string = string.replace(code, char.encode('utf-8'))
+            string = string.replace(code, char)
         string = string.replace("\\n", "\n").replace("\\t", "\t")
     return string
 
 
 class WordfastDialect(csv.Dialect):
     """Describe the properties of a Wordfast generated TAB-delimited file."""
+
     delimiter = "\t"
     lineterminator = "\r\n"
     quoting = csv.QUOTE_NONE
+
+
 csv.register_dialect("wordfast", WordfastDialect)
 
 
-class WordfastTime(object):
+class WordfastTime:
     """Manages time stamps in the Wordfast format of YYYYMMDD~hhmmss"""
 
     def __init__(self, newtime=None):
         self._time = None
         if not newtime:
             self.time = None
-        elif isinstance(newtime, basestring):
+        elif isinstance(newtime, str):
             self.timestring = newtime
         elif isinstance(newtime, time.struct_time):
             self.time = newtime
@@ -218,6 +245,7 @@ class WordfastTime(object):
         :type timestring: String
         """
         self._time = time.strptime(timestring, WF_TIMEFORMAT)
+
     timestring = property(get_timestring, set_timestring)
 
     def get_time(self):
@@ -234,6 +262,7 @@ class WordfastTime(object):
             self._time = newtime
         else:
             self._time = None
+
     time = property(get_time, set_time)
 
     def __str__(self):
@@ -243,7 +272,7 @@ class WordfastTime(object):
             return self.timestring
 
 
-class WordfastHeader(object):
+class WordfastHeader:
     """A wordfast translation memory header"""
 
     def __init__(self, header=None):
@@ -255,9 +284,11 @@ class WordfastHeader(object):
 
     def _create_default_header(self):
         """Create a default Wordfast header with the date set to the current
-        time"""
-        defaultheader = WF_FIELDNAMES_HEADER_DEFAULTS
-        defaultheader['date'] = '%%%s' % WordfastTime(time.localtime()).timestring
+        time
+        """
+        defaultheader = {}
+        defaultheader.update(WF_FIELDNAMES_HEADER_DEFAULTS)
+        defaultheader["date"] = "%%%s" % WordfastTime(time.localtime()).timestring
         return defaultheader
 
     def getheader(self):
@@ -266,14 +297,17 @@ class WordfastHeader(object):
 
     def setheader(self, newheader):
         self._header_dict = newheader
+
     header = property(getheader, setheader)
 
     def settargetlang(self, newlang):
-        self._header_dict['target-lang'] = '%%%s' % newlang
+        self._header_dict["target-lang"] = "%%%s" % newlang
+
     targetlang = property(None, settargetlang)
 
     def settucount(self, count):
-        self._header_dict['tucount'] = '%%TU=%08d' % count
+        self._header_dict["tucount"] = "%%TU=%08d" % count
+
     tucount = property(None, settucount)
 
 
@@ -284,11 +318,11 @@ class WordfastUnit(base.TranslationUnit):
         self._dict = {}
         if source:
             self.source = source
-        super(WordfastUnit, self).__init__(source)
+        super().__init__(source)
 
     def _update_timestamp(self):
         """Refresh the timestamp for the unit"""
-        self._dict['date'] = WordfastTime(time.localtime()).timestring
+        self._dict["date"] = WordfastTime(time.localtime()).timestring
 
     def getdict(self):
         """Get the dictionary of values for a Wordfast line"""
@@ -302,122 +336,127 @@ class WordfastUnit(base.TranslationUnit):
         """
         # TODO First check that the values are OK
         self._dict = newdict
+
     dict = property(getdict, setdict)
 
     def _get_source_or_target(self, key):
         if self._dict.get(key, None) is None:
             return None
         elif self._dict[key]:
-            return _wf_to_char(self._dict[key]).decode('utf-8')
+            return _wf_to_char(self._dict[key])
         else:
             return ""
 
     def _set_source_or_target(self, key, newvalue):
         if newvalue is None:
             self._dict[key] = None
-        if isinstance(newvalue, unicode):
-            newvalue = newvalue.encode('utf-8')
         newvalue = _char_to_wf(newvalue)
-        if not key in self._dict or newvalue != self._dict[key]:
+        if key not in self._dict or newvalue != self._dict[key]:
             self._dict[key] = newvalue
             self._update_timestamp()
 
-    def getsource(self):
-        return self._get_source_or_target('source')
+    @property
+    def source(self):
+        return self._get_source_or_target("source")
 
-    def setsource(self, newsource):
+    @source.setter
+    def source(self, source):
         self._rich_source = None
-        return self._set_source_or_target('source', newsource)
-    source = property(getsource, setsource)
+        self._set_source_or_target("source", source)
 
-    def gettarget(self):
-        return self._get_source_or_target('target')
+    @property
+    def target(self):
+        return self._get_source_or_target("target")
 
-    def settarget(self, newtarget):
+    @target.setter
+    def target(self, target):
         self._rich_target = None
-        return self._set_source_or_target('target', newtarget)
-    target = property(gettarget, settarget)
+        self._set_source_or_target("target", target)
 
     def settargetlang(self, newlang):
-        self._dict['target-lang'] = newlang
+        self._dict["target-lang"] = newlang
+
     targetlang = property(None, settargetlang)
 
     def __str__(self):
         return str(self._dict)
 
     def istranslated(self):
-        if not self._dict.get('source', None):
+        if not self._dict.get("source", None):
             return False
-        return bool(self._dict.get('target', None))
+        return bool(self._dict.get("target", None))
 
 
 class WordfastTMFile(base.TranslationStore):
     """A Wordfast translation memory file"""
-    Name = _("Wordfast Translation Memory")
+
+    Name = "Wordfast Translation Memory"
     Mimetypes = ["application/x-wordfast"]
     Extensions = ["txt"]
+    UnitClass = WordfastUnit
+    default_encoding = "iso-8859-1"
 
-    def __init__(self, inputfile=None, unitclass=WordfastUnit):
+    def __init__(self, inputfile=None, **kwargs):
         """construct a Wordfast TM, optionally reading in from inputfile."""
-        self.UnitClass = unitclass
-        base.TranslationStore.__init__(self, unitclass=unitclass)
-        self.filename = ''
+        super().__init__(**kwargs)
+        self.filename = ""
         self.header = WordfastHeader()
-        self._encoding = 'iso-8859-1'
         if inputfile is not None:
             self.parse(inputfile)
 
     def parse(self, input):
         """parsese the given file or file source string"""
-        if hasattr(input, 'name'):
+        if hasattr(input, "name"):
             self.filename = input.name
-        elif not getattr(self, 'filename', ''):
-            self.filename = ''
+        elif not getattr(self, "filename", ""):
+            self.filename = ""
         if hasattr(input, "read"):
             tmsrc = input.read()
             input.close()
             input = tmsrc
-        if TAB_UTF16 in input.split("\n")[0]:
-            self._encoding = 'utf-16'
+        if TAB_UTF16 in input.split(b"\n")[0]:
+            self.encoding = "utf-16"
         else:
-            self._encoding = 'iso-8859-1'
+            self.encoding = "iso-8859-1"
         try:
-            input = input.decode(self._encoding).encode('utf-8')
-        except:
-            raise ValueError("Wordfast files are either UTF-16 (UCS2) or ISO-8859-1 encoded")
-        for header in csv.DictReader(input.split("\n")[:1],
-                                    fieldnames=WF_FIELDNAMES_HEADER,
-                                    dialect="wordfast"):
-            self.header = WordfastHeader(header)
-        lines = csv.DictReader(input.split("\n")[1:],
-                                fieldnames=WF_FIELDNAMES,
-                                dialect="wordfast")
-        for line in lines:
+            input = input.decode(self.encoding)
+        except Exception:
+            raise ValueError(
+                "Wordfast files are either UTF-16 (UCS2) or ISO-8859-1 encoded"
+            )
+        reader = csv.DictReader(
+            input.split("\n"), fieldnames=WF_FIELDNAMES, dialect="wordfast"
+        )
+        for idx, line in enumerate(reader):
+            if idx == 0:
+                header = dict(
+                    zip(WF_FIELDNAMES_HEADER, [line[key] for key in WF_FIELDNAMES])
+                )
+                self.header = WordfastHeader(header)
+                continue
             newunit = WordfastUnit()
             newunit.dict = line
             self.addunit(newunit)
 
-    def __str__(self):
+    def serialize(self, out):
+        # Check first if there is at least one translated unit
+        translated_units = [u for u in self.units if u.istranslated()]
+        if not translated_units:
+            return
+
         output = csv.StringIO()
-        header_output = csv.StringIO()
-        writer = csv.DictWriter(output, fieldnames=WF_FIELDNAMES,
-                                dialect="wordfast")
-        unit_count = 0
-        for unit in self.units:
-            if unit.istranslated():
-                unit_count += 1
-                writer.writerow(unit.dict)
-        if unit_count == 0:
-            return ""
-        output.reset()
-        self.header.tucount = unit_count
-        outheader = csv.DictWriter(header_output,
-                                    fieldnames=WF_FIELDNAMES_HEADER,
-                                    dialect="wordfast")
-        outheader.writerow(self.header.header)
-        header_output.reset()
-        decoded = "".join(header_output.readlines() + output.readlines()).decode('utf-8')
-        try:
-            return decoded.encode(self._encoding)
-        except UnicodeEncodeError:
-            return decoded.encode('utf-16')
+        writer = csv.DictWriter(output, fieldnames=WF_FIELDNAMES, dialect="wordfast")
+        # No real headers, the first line contains metadata
+        self.header.tucount = len(translated_units)
+        writer.writerow(
+            dict(
+                zip(
+                    WF_FIELDNAMES,
+                    [self.header.header[key] for key in WF_FIELDNAMES_HEADER],
+                )
+            )
+        )
+
+        for unit in translated_units:
+            writer.writerow(unit.dict)
+        out.write(output.getvalue().encode(self.encoding))
